!MNH_LIC Copyright 2005-2019 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!    ################
     MODULE MODI_SBL_DEPTH  
!    ################ 
!
INTERFACE
!
      SUBROUTINE SBL_DEPTH(KKB,KKE,PZZ,PFLXU,PFLXV,PWTHV,PLMO,PSBL_DEPTH)
!
INTEGER,                INTENT(IN)    :: KKB       ! first physical level
INTEGER,                INTENT(IN)    :: KKE       ! upper physical level
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PZZ       ! altitude of flux levels
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PFLXU     ! u'w'
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PFLXV     ! v'w'
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PWTHV     ! buoyancy flux
REAL, DIMENSION(:,:),   INTENT(IN)    :: PLMO      ! Monin-Obukhov length
REAL, DIMENSION(:,:),   INTENT(INOUT) :: PSBL_DEPTH! boundary layer height
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE SBL_DEPTH
!
END INTERFACE
!
END MODULE MODI_SBL_DEPTH
!
!     #################################################################
      SUBROUTINE SBL_DEPTH(KKB,KKE,PZZ,PFLXU,PFLXV,PWTHV,PLMO,PSBL_DEPTH)
!     #################################################################
!
!
!!****  *SBL_DEPTH* - computes SBL depth
!!
!!    PURPOSE
!!    -------
!
!!**  METHOD
!!    ------
!!
!!    SBL is defined as the layer where momentum flux is equal to XSBL_FRAC of its surface value
!!    
!!    EXTERNAL
!!    --------
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!      V. Masson  * Meteo-France *
!!
!!    MODIFICATIONS
!!    -------------
!!      Original         nov. 2005
!!      26/02/2020   T.Nagel Correction of SBL depth computation in neutral stratification
!! --------------------------------------------------------------------------
!       
!*      0. DECLARATIONS
!          ------------
!
USE MODD_PARAMETERS, ONLY : XUNDEF
USE MODD_CTURB,      ONLY : XFTOP_O_FSURF, XSBL_O_BL

use mode_mppdb
#ifdef MNH_OPENACC
use mode_msg
#endif

USE MODI_BL_DEPTH_DIAG
!
IMPLICIT NONE
!
!
!*      0.1  declarations of arguments
!
INTEGER,                INTENT(IN)    :: KKB       ! first physical level
INTEGER,                INTENT(IN)    :: KKE       ! upper physical level
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PZZ       ! altitude of flux levels
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PFLXU     ! u'w'
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PFLXV     ! v'w'
REAL, DIMENSION(:,:,:), INTENT(IN)    :: PWTHV     ! buoyancy flux
REAL, DIMENSION(:,:),   INTENT(IN)    :: PLMO      ! Monin-Obukhov length
REAL, DIMENSION(:,:),   INTENT(INOUT) :: PSBL_DEPTH! boundary layer height
!
!-------------------------------------------------------------------------------
!
!       0.2  declaration of local variables
!
!
INTEGER                             :: JLOOP    ! loop counter
REAL, DIMENSION(:,:),   allocatable :: ZQ0      ! surface buoyancy flux
REAL, DIMENSION(:,:),   allocatable :: ZWU      ! surface friction u'w'
REAL, DIMENSION(:,:),   allocatable :: ZWV      ! surface friction v'w'
REAL, DIMENSION(:,:),   allocatable :: ZUSTAR2  ! surface friction
REAL, DIMENSION(:,:),   allocatable :: ZSBL_DYN ! SBL wih dynamical criteria
REAL, DIMENSION(:,:),   allocatable :: ZSBL_THER! SBL wih thermal   criteria
REAL, DIMENSION(:,:),   allocatable :: ZA       ! ponderation coefficient
REAL, DIMENSION(:,:,:), allocatable :: ZWIND    ! intermediate wind for SBL calculation
#ifdef MNH_OPENACC
call Print_msg( NVERB_FATAL, 'GEN', 'SBL_DEPTH', 'not yet implemented' )
#endif
!----------------------------------------------------------------------------

if ( mppdb_initialized ) then
  !Check all in arrays
  call Mppdb_check( pzz,   "Sbl_depth beg:pzz"   )
  call Mppdb_check( pflxu, "Sbl_depth beg:pflxu" )
  call Mppdb_check( pflxv, "Sbl_depth beg:pflxv" )
  call Mppdb_check( pwthv, "Sbl_depth beg:pwthv" )
  call Mppdb_check( plmo,  "Sbl_depth beg:plmo"  )
  !Check all inout arrays
  call Mppdb_check( psbl_depth, "Sbl_depth beg:psbl_depth" )
end if

allocate( zq0      (size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( zwu      (size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( zwv      (size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( zustar2  (size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( zsbl_dyn (size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( zsbl_ther(size( pzz, 1 ), size( pzz, 2 ) ) )
allocate( za       (size( pzz, 1 ), size( pzz, 2 ) ) )

allocate( zwind    (size( pflxu, 1 ), size( pflxu, 2 ), size( pflxu, 3 ) ) )
!
!* initialisations
!
!
ZWU (:,:) = PFLXU(:,:,KKB)
ZWV (:,:) = PFLXV(:,:,KKB)
ZQ0 (:,:) = PWTHV(:,:,KKB)
!
ZUSTAR2(:,:) = SQRT(ZWU**2+ZWV**2)
!
!----------------------------------------------------------------------------
!
!* BL and SBL diagnosed with friction criteria
!
ZWIND=SQRT(PFLXU**2+PFLXV**2)
ZSBL_DYN = XSBL_O_BL * BL_DEPTH_DIAG(KKB,KKE,ZUSTAR2,PZZ(:,:,KKB),ZWIND,PZZ,XFTOP_O_FSURF) 
!
!----------------------------------------------------------------------------
!
!* BL and SBL diagnosed with buoyancy flux criteria
!
ZSBL_THER= XSBL_O_BL * BL_DEPTH_DIAG(KKB,KKE,ZQ0,PZZ(:,:,KKB),PWTHV,PZZ,XFTOP_O_FSURF)
!
!----------------------------------------------------------------------------
!
!* SBL depth
!
PSBL_DEPTH = 0.
WHERE (ZSBL_THER> 0. .AND. ZSBL_DYN> 0.) PSBL_DEPTH = MIN(ZSBL_THER(:,:),ZSBL_DYN(:,:))
WHERE (ZSBL_THER> 0. .AND. ZSBL_DYN==0.) PSBL_DEPTH = ZSBL_THER(:,:)
WHERE (ZSBL_THER==0. .AND. ZSBL_DYN> 0.) PSBL_DEPTH = ZSBL_DYN(:,:)
!
DO JLOOP=1,5
  WHERE (PLMO(:,:)/=XUNDEF .AND. ABS(PLMO(:,:))>=0.01 )
    ZA = TANH(2.*PSBL_DEPTH/PLMO)**2
    PSBL_DEPTH = 0.2 * PSBL_DEPTH + 0.8 * ((1.-ZA) * ZSBL_DYN + ZA * ZSBL_THER )
  END WHERE
END DO
WHERE (ABS(PLMO(:,:))<=0.01 )     PSBL_DEPTH = ZSBL_THER
WHERE (PLMO(:,:)==XUNDEF) PSBL_DEPTH = ZSBL_DYN

if ( mppdb_initialized ) then
  !Check all inout arrays
  call Mppdb_check( psbl_depth, "Sbl_depth end:psbl_depth" )
end if

!----------------------------------------------------------------------------
END SUBROUTINE SBL_DEPTH
